unit fmGUI;
{ --------------------------------------------------------------------------------
    Name: fmGUIdiff.pas
  Author: Scott Mattes
    Date: 2/13/98
 Purpose: To provide a GUI front-end to the diff program of S.Kurinny & S.Kostinsky,
          kurinn@gu.kiev.ua


  Modification History:
  ---------------------
  yyyy-mm-dd your name, your organization
       text describing what you changed and why

  1998-02-18 Sergey Kurinny, Dream Company
             Set Scaled=False in all project forms
             added search path to project options
             Set enabled=False on BarStatus

------------------------------------------------------------------------------- }
                                  
interface

uses
  Buttons,
  Classes,
  ComCtrls,
  Controls,
  Dialogs,
  ExtCtrls,
  Forms,
  Graphics,
  Menus,
  Messages,
  Spin,
  StdCtrls,
  SysUtils,
  Windows;

type
  TfrmGUIdiff = class(TForm)
    btnApply: TButton;
    btnMake: TButton;
    btnOpenBase: TSpeedButton;
    btnOpenCompare: TSpeedButton;
    btnOpenDiff: TSpeedButton;
    btnQuit: TButton;
    dlgOpen: TOpenDialog;
    edtBase: TEdit;
    edtCompare: TEdit;
    edtDiff: TEdit;
    grpOperation: TGroupBox;
    lblBase: TLabel;
    lblCompare: TLabel;
    lblDiff: TLabel;
    lblLevel: TLabel;
    lblWorking: TLabel;
    mniAbout: TMenuItem;
    mniContents: TMenuItem;
    mniExit: TMenuItem;
    mniFile: TMenuItem;
    mniHelp: TMenuItem;
    mnuMain: TMainMenu;
    pnlMain: TPanel;
    radApply: TRadioButton;
    radMake: TRadioButton;
    sepOne: TMenuItem;
    spnLevel: TSpinEdit;
    BarStatus: TStatusBar;

    procedure btnApplyClick(Sender: TObject);
    procedure btnMakeClick(Sender: TObject);
    procedure btnOpenBaseClick(Sender: TObject);
    procedure btnOpenCompareClick(Sender: TObject);
    procedure btnOpenDiffClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure edtBaseChange(Sender: TObject);
    procedure edtCompareChange(Sender: TObject);
    procedure edtDiffChange(Sender: TObject);
    procedure mniAboutClick(Sender: TObject);
    procedure mniContentsClick(Sender: TObject);
    procedure mniExitClick(Sender: TObject);
    procedure radApplyClick(Sender: TObject);
    procedure radMakeClick(Sender: TObject);

  private
    { Private declarations }
    procedure CheckToEraseFile;
    procedure CloseStreams;
    procedure CompareFileNames;
    procedure EnableButtons;
    procedure OpenStreams(parm2: string; parm3: string; parm4: string);
    procedure StatusMsg(msg: string);

  public
    { Public declarations }
  end;

var
  frmGUIdiff: TfrmGUIdiff;
  InStream,OutStream,UseStream: TFileStream;
  maxlev: integer;

implementation

uses aDiff,
     fmAbout,
     fmCont;

{$R *.DFM}



{---------------------------------------------------------------------------------
        Name: NotifyMe
     Purpose: To let us know that the program is still working
       Notes: Called by aDiff every once in a while
 -------------------------------------------------------------------------------- }
procedure NotifyMe;
begin

  with frmGUIdiff do
  begin
    lblWorking.Visible := (lblWorking.Visible = false);
    lblWorking.Update;
  end;    //with

end;    // NotifyMe
//
//
//



{---------------------------------------------------------------------------------
        Name: btnApplyClick
     Purpose: To run aDiff to apply a diff file to a base file and create the new
              file
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.btnApplyClick(Sender: TObject);
begin

  try
    Screen.Cursor := crHourGlass;
    StatusMsg('Applying compressed file...');
    OpenStreams(edtDiff.text, edtCompare.text, edtBase.text);
    DiffStreamExtract(InStream,UseStream,OutStream,NotifyMe);
  finally
    CloseStreams;
    StatusMsg('Done');
    Screen.Cursor := crDefault;
  end;  { try/finally }

end;      // TfrmGUIdiff.btnApplyClick
//
//
//



{---------------------------------------------------------------------------------
        Name: btnMakeClick
     Purpose: To run aDiff to create a diff file
 -------------------------------------------------------------------------------- }                      
procedure TfrmGUIdiff.btnMakeClick(Sender: TObject);
begin

  try
    Screen.Cursor := crHourGlass;
    StatusMsg('Making compressed file...');
    MaxLev := spnLevel.value;
    OpenStreams(edtCompare.text, edtDiff.text, edtBase.text);
    DiffStreamCompress(InStream,UseStream,OutStream,NotifyMe,MaxLev);
  finally
    CloseStreams;
    StatusMsg('Done');
    Screen.Cursor := crDefault;
  end;  { try/finally }

end;      // TfrmGUIdiff.btnMakeClick
//
//
//



{---------------------------------------------------------------------------------
        Name: btnOpenBaseClick
     Purpose: To get the full path and file name of the base file
       Notes: Saves the path to the file as the start point for further opens
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.btnOpenBaseClick(Sender: TObject);
begin

   if dlgOpen.Execute
   then begin
          edtBase.text := dlgOpen.FileName;
          dlgOpen.InitialDir := ExtractFilePath(edtBase.text);
        end;

   EnableButtons;

end;      // TfrmGUIdiff.btnOpenBaseClick
//
//
//



{---------------------------------------------------------------------------------
        Name: btnOpenCompareClick
     Purpose: To get the full file path and name of the compare file if making a
              diff file and the name of the file to create if applying a diff
       Notes: Save the path for further open requests
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.btnOpenCompareClick(Sender: TObject);
begin

   if dlgOpen.Execute
   then begin
          if radApply.checked
          then CheckToEraseFile;

          edtCompare.text := dlgOpen.FileName;
          dlgOpen.InitialDir := ExtractFilePath(edtCompare.text);
        end;

   EnableButtons;

end;      // TfrmGUIdiff.btnOpenCompareClick
//
//
//



{---------------------------------------------------------------------------------
        Name: btnOpenDiffClick
     Purpose: To get the full path and name of the diff file
       Notes: Save the path for further open requests
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.btnOpenDiffClick(Sender: TObject);
begin

   if dlgOpen.Execute
   then begin
          if radMake.checked
          then CheckToEraseFile;

          edtDiff.text := dlgOpen.FileName;
          dlgOpen.InitialDir := ExtractFilePath(edtDiff.text);
        end;

   EnableButtons;

end;      // TfrmGUIdiff.btnOpenDiffClick
//
//
//



{--------------------------------------------------------------------------------}
procedure TfrmGUIdiff.btnQuitClick(Sender: TObject);
begin

  close;

end;      // TfrmGUIdiff.btnQuitClick
//
//
//



{---------------------------------------------------------------------------------
        Name: CheckToEraseFile
     Purpose: After the user specifies a file, check if it exists and ask if it
              should be erased first
       Notes: Called to check on either the diff file if making a diff and for a
              new file if applying a diff file
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.CheckToEraseFile;
begin

  if FileExists(dlgOpen.FileName)
  then if MessageDlg('File ' + dlgOpen.FileName + ' already exists!'+#13+#10+'Do you wish to erase it?',mtWarning,[mbYes,mbNo],0) = mrYes
// reference DeleteFile with the unit it is in because some other unit also has
// a DeleteFile and it has different parameters. 98/02/13 Scott Mattes
       then SysUtils.DeleteFile(dlgOpen.FileName);

end;      // TfrmGUIdiff.CheckToEraseFile
//
//
//



{---------------------------------------------------------------------------------
        Name: CloseStreams
     Purpose: To clean up memory after ourselves
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.CloseStreams;
begin

  InStream.Free;
  OutStream.Free;
  UseStream.Free;

end;      // TfrmGUIdiff.CloseStreams
//
//
//



{---------------------------------------------------------------------------------
        Name: CompareFileNames
     Purpose: To make sure the user doesn't use the same file name more than once
       Notes: None of the three file names can be the same!
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.CompareFileNames;
var
  AllFileNames: string;

begin

  if (pos(edtDiff.text, AllFileNames) > 0) or
     (pos(edtCompare.text, AllFileNames) > 0 ) or
     (pos(edtBase.text, AllFileNames) > 0)
  then begin
         MessageDlg('None of the files can be the same!',mtError,[mbOK],0);
         abort;
       end;

end;    // TfrmGUIdiff.CompareFileNames;
//
//
//



{---------------------------------------------------------------------------------
        Name: edtBaseChange
     Purpose: To check if there is a problem with the file name selected and if
              not, and all the file names are present, let them press the go button
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.edtBaseChange(Sender: TObject);
begin

   CompareFileNames;
   EnableButtons;

end;      // TfrmGUIdiff.edtBaseChange
//
//
//



{---------------------------------------------------------------------------------
        Name: edtCompareChange
     Purpose: To check if there is a problem with the file name selected and if
              not, and all the file names are present, let them press the go button
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.edtCompareChange(Sender: TObject);
begin

   CompareFileNames;
   EnableButtons;

end;      // TfrmGUIdiff.edtCompareChange
//
//
//



{---------------------------------------------------------------------------------
        Name: edtDiffChange
     Purpose: To check if there is a problem with the file name selected and if
              not, and all the file names are present, let them press the go button
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.edtDiffChange(Sender: TObject);
begin

   CompareFileNames;
   EnableButtons;

end;      // TfrmGUIdiff.edtDiffChange
//
//
//



{---------------------------------------------------------------------------------
        Name: EnableButtons
     Purpose: To enable the Make/Apply button when all the fields are filled in
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.EnableButtons;
begin

  if (edtBase.text <> '') and
     (edtCompare.text <> '') and
     (edtDiff.text <> '')
  then begin
         btnMake.enabled := true;
         btnApply.enabled := true;
       end
  else begin
         btnMake.enabled := false;
         btnApply.enabled := false;
       end;

end;    // EnableButtons
//
//
//



{---------------------------------------------------------------------------------
        Name: mniAboutClick
     Purpose: To show the user the about box for this thing
       Notes: Create and dispose of the about screen here
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.mniAboutClick(Sender: TObject);
begin

  with TfrmAbout.create(application) do
  begin
    try
      frmGUIdiff.hide;
      showmodal;
    finally //wrap up
      free;
      frmGUIdiff.show;
    end;    //try / finally
  end;    //with

end;      // TfrmGUIdiff.mniAboutClick
//
//
//



{---------------------------------------------------------------------------------
        Name: mniContentsClick
     Purpose: To show the user the pitifully little help there is for this thing
       Notes: Create and dispose of the content screen here
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.mniContentsClick(Sender: TObject);
begin

  with TfrmContents.create(application) do
  begin
    try
      frmGUIdiff.hide;
      showmodal;
    finally //wrap up
      free;
      frmGUIdiff.show;
    end;    //try / finally
  end;    //with

end;      // TfrmGUIdiff.mniContentsClick
//
//
//



{--------------------------------------------------------------------------------}
procedure TfrmGUIdiff.mniExitClick(Sender: TObject);
begin

  close;

end;
//
//
//



{---------------------------------------------------------------------------------
        Name: OpenStreams
  Parameters: The full path & names of the InStream, OutStream, UseStream files
     Purpose: To create the streams needed by aDiff
        Note: Uses the same stream names as diff.dpr
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.OpenStreams(parm2: string; parm3: string; parm4: string);
begin

  InStream:=TFileStream.Create(parm2,fmopenread+fmShareDenyNone);
  OutStream:=TFileStream.Create(parm3,fmcreate);
  UseStream:=TFileStream.Create(parm4,fmopenread+fmShareDenyNone);

end;      // TfrmGUIdiff.OpenStreams
//
//
//



{---------------------------------------------------------------------------------
        Name: radApplyClick
     Purpose: To switch which button is visible, to make sure the buttons are set
              right for the operation and to make sure the user knows which file
              to put in for the compare file
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.radApplyClick(Sender: TObject);
begin

  btnMake.visible := false;
  btnApply.visible := true;

  EnableButtons;

  lblCompare.caption := 'File to create';

end;      // TfrmGUIdiff.radApplyClick
//
//
//



{---------------------------------------------------------------------------------
        Name: radMakeClick
     Purpose: To switch which button is visible, to make sure the buttons are set
              right for the operation and to make sure the user knows which file
              to put in for the compare file
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.radMakeClick(Sender: TObject);
begin

  btnMake.visible := true;
  btnApply.visible := false;

  EnableButtons;

  lblCompare.caption := 'File to compare to';

end;      // TfrmGUIdiff.radMakeClick
//
//
//



{---------------------------------------------------------------------------------
        Name: StatusMsg
  Parameters: The message to be displayed
     Purpose: To display a message on the status bar
       Notes: Update the status bar so that the user sees the message
 -------------------------------------------------------------------------------- }
procedure TfrmGUIdiff.StatusMsg(msg: string);
begin

  with barStatus do
  begin
    simpletext := msg;
    update;
  end;    //with

end;      // TfrmGUIdiff.StatusMsg
//
//
//



end.
